// L138_iirsosadapt_intr.c 
//

#include "L138_aic3106_init.h"
#include "elliptic.cof"            // IIR filter coefficients

float w[NUM_SECTIONS][2] = {0.0, 0.0, 0.0, 0.0};
float beta = 1E-13;          	         //rate of convergence
#define WLENGTH 256                     //# of coefffor adaptive FIR
float weights[WLENGTH+1];		         //buffer coeff for adaptive FIR
float x[WLENGTH+1];	         //buffer samples of adaptive FIR
AIC31_data_type codec_data;

interrupt void interrupt4(void)               //ISR
{
 int i;
 float adaptfir_out;                 //init output of adapt FIR
 float error;                              //error=diff of fixed/adapt out

  int section;   // index for section number
  float input;   // input to each section
  float wn,yn;   // intermediate and output values in each stage
 input = (float)(prbs());
 x[0]=input;          	   //as well as to adaptive FIR
  
  for (section=0 ; section< NUM_SECTIONS ; section++)
  {
    wn = input - a[section][1]*w[section][0] - a[section][2]*w[section][1];
    yn = b[section][0]*wn + b[section][1]*w[section][0] + b[section][2]*w[section][1];
    w[section][1] = w[section][0];
    w[section][0] = wn;
    input = yn;              // output of current section will be input to next
  }

 adaptfir_out = 0.0;
 for (i = 0; i < WLENGTH; i++)
   adaptfir_out +=(weights[i]*x[i]); //adaptive FIR filter output

 error = yn - adaptfir_out;           //error signal

 for (i = WLENGTH-1; i >= 0; i--)
  {
   weights[i] = weights[i]+(beta*error*x[i]);  //update weights of adaptive FIR
   x[i+1] = x[i];      //update samples of adaptive FIR
  }
  codec_data.channel[RIGHT]= ((uint16_t)(adaptfir_out));
  codec_data.channel[LEFT]= ((uint16_t)(error));
  output_sample(codec_data.uint);
 return;
}

int main()
{
 int i;

 for (i = 0; i < WLENGTH; i++)
  {
   weights[i] = 0.0;       			   //init coeff for adaptive FIR
   x[i] = 0.0;			   //init buffer for adaptive FIR
  }
//w[0][0] = 0.0;
//w[0][1] = 0.0;
//w[1][0] = 0.0;
//w[0][1] = 0.0;

 L138_initialise_intr(FS_8000_HZ,ADC_GAIN_0DB,DAC_ATTEN_0DB);

 while(1);
 

}
