// L138_bios_sysid_edma_TSK.c
//

#include <std.h>
#include <sem.h>
#include "L138_bios_aic3106_init.h"

extern int16_t *pingIN, *pingOUT, *pongIN, *pongOUT;
SEM_Handle SEM_buffer_full;
int procBuffer;

#define beta 1E-12            // learning rate
#define WLENGTH 256

float w[WLENGTH+1];           // adaptive filter weights
float x[WLENGTH+1];	          // delay line

void interrupt4(void)         // interrupt service routine
{
 switch(EDMA_3CC_IPR)
  {
    case 1:                     // TCC = 0
      procBuffer = PING;        // process ping
	  EDMA_3CC_ICR = 0x0001;    // clear EDMA3 IPR bit TCC
	  break;
    case 2:                     // TCC = 1
      procBuffer = PONG;        // process pong
	  EDMA_3CC_ICR = 0x0002;    // clear EDMA3 IPR bit TCC 
	  break;
    default:                    // may have missed an interrupt
	  EDMA_3CC_ICR = 0x0003;    // clear EDMA3 IPR bits 0 and 1
      break;
  }
  EVTCLR0 = 0x00000100;
  SEM_postBinary(SEM_buffer_full);
  return;
}


void process_buffer(void)
{
  int16_t *inBuf, *outBuf;      // pointers to process buffers
  int16_t left_sample, right_sample;
  int i,j;
  float d,y,e;

  if (procBuffer == PING)
  {
    inBuf = pingIN;
    outBuf = pingOUT;
  }
  if (procBuffer == PONG)
  {
    inBuf = pongIN;
    outBuf = pongOUT;
  }
  for (i = 0; i < (BUFCOUNT/2) ; i++) 
  {
    left_sample = *inBuf++;
    right_sample = *inBuf++;

    d = (float)(left_sample);
	x[0] = prbs();
    left_sample = (int16_t)(x[0]);

    *outBuf++ = left_sample;
    *outBuf++ = right_sample;

    y = 0.0;
    for (j = 0; j < WLENGTH; j++)
      y +=(w[j]*x[j]);             //compute filter output 
 
    e = d - y;                     // error signal           
 
    for (j = WLENGTH-1; j >= 0; j--)         
    {
       w[j] = w[j]+(beta*e*x[j]); // update weights
       x[j+1] = x[j];             // update delay line   
    } 
  }
  return;
}

void endless_task(void)
{
  int status;

  while(1)
  {
    status = SEM_pendBinary(SEM_buffer_full, SYS_FOREVER);
    process_buffer();
  }
}

int main(void)
{
  int i;

  for (i=0; i <= WLENGTH; i++)
  {
    w[i] = 0.0;       			   //init coeff for adaptive FIR
    x[i] = 0.0;			   //init buffer for adaptive FIR
  }  
  SEM_buffer_full = SEM_create(0, NULL);
  L138_bios_initialise_edma(FS_8000_HZ,ADC_GAIN_0DB,DAC_ATTEN_0DB);
}


