// L138_psp_utils.c
//

#include "L138_psp_utils.h"

Uint32 prand_seed = 1;

typedef struct BITVAL    //register bits to be packed as integer
{
 unsigned int b0:1, b1:1, b2:1, b3:1, b4:1, b5:1, b6:1;
 unsigned int b7:1, b8:1, b9:1, b10:1, b11:1, b12:1,b13:1;
 unsigned int dweebie:2; //Fills the 2 bit hole - bits 14-15
} bitval;

typedef union SHIFT_REG
{
 unsigned int regval;
 bitval bt;
} shift_reg;
int fb = 1;                            //feedback variable
shift_reg sreg = {0xFFFF};                    //shift register

Uint32 rand31_next()
{
  Uint32 hi, lo;

  lo = 16807 * (prand_seed & 0xFFFF);
  hi = 16807 * (prand_seed >> 16);

  lo += (hi & 0x7FFF) << 16;
  lo += hi >> 15;

  if (lo > 0x7FFFFFFF) lo -= 0x7FFFFFFF;

  return(prand_seed = (Uint32)lo);
}

Int16 prand()
{
return ((Int16)(rand31_next()>>18)-4096);
}

void L138_AIC3106_writeReg(SIO_Handle ioStream, Uint32 reg_addr, Uint32 reg_data)
{
ICodec_RegData mydata;
int DSR_status;
Audio_IoctlParam ioctlPrm;
Uint32 DSR_reg_data;

DSR_reg_data = reg_data;
mydata.regIndex = reg_addr;
mydata.regData = (Ptr)&DSR_reg_data;
mydata.regCount = 1;
ioctlPrm.aiModule = Audio_ModuleSel_AUDIO_CODEC;
ioctlPrm.codecId = 0;
ioctlPrm.ioctlArg = (Ptr)&mydata;
DSR_status = SIO_ctrl(ioStream, Aic31_AC_IOCTL_REG_WRITE, (Arg)&ioctlPrm);
if(DSR_status) 	mydata.regCount = 2;
}

int prbs(void) 			       //gen pseudo-random sequence {-1,1}
{
  int prnseq;			 
  if(sreg.bt.b0)
	prnseq = -NOISELEVEL;		           //scaled negative noise level
  else
	prnseq = NOISELEVEL;		           //scaled positive noise level
  fb =(sreg.bt.b0)^(sreg.bt.b1);   //XOR bits 0,1
  fb^=(sreg.bt.b11)^(sreg.bt.b13); //with bits 11,13 -> fb
  sreg.regval<<=1;
  sreg.bt.b0=fb;			       //close feedback path
  return prnseq;			       //return noise sequence value 
}

/* generate real and imaginary twiddle table of size n/2 complex numbers */
void gen_w_r2(float* w, int n)
{
    int i;
    float pi = 4.0 * atan(1.0);
    float e = pi * 2.0 / n;

    for(i = 0; i < (n >> 1); i++)
    {
        w[2 * i] = cos(i * e);
        w[2 * i + 1] = sin(i * e);
    }
}

/* bit reverse coefficients */
void bit_rev(float* x, int n)
{
    int i, j, k;
    float rtemp, itemp;
    j = 0;

    for(i=1; i < (n-1); i++)
    {
        k = n >> 1;

        while(k <= j)
        {
            j -= k;
            k >>= 1;
        }

        j += k;

        if(i < j)
        {
            rtemp = x[j*2];
            x[j*2] = x[i*2];
            x[i*2] = rtemp;
            itemp = x[j*2+1];
            x[j*2+1] = x[i*2+1];
            x[i*2+1] = itemp;
        }
    }
}
